package com.ym.mec.biz.service.impl;

import static com.ym.mec.biz.dal.enums.DealStatusEnum.SUCCESS;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.ym.mec.biz.dal.dao.MusicGroupDao;
import com.ym.mec.biz.dal.dao.MusicGroupPaymentCalenderCourseSettingsDao;
import com.ym.mec.biz.dal.dao.MusicGroupPurchaseListDao;
import com.ym.mec.biz.dal.dao.OrganizationDao;
import com.ym.mec.biz.dal.dao.StudentPaymentOrderDao;
import com.ym.mec.biz.dal.dao.StudentPaymentOrderDetailDao;
import com.ym.mec.biz.dal.dao.StudentRegistrationDao;
import com.ym.mec.biz.dal.dao.SubjectChangeDao;
import com.ym.mec.biz.dal.dao.SubjectDao;
import com.ym.mec.biz.dal.dao.TeacherDao;
import com.ym.mec.biz.dal.dto.MusicalListDetailDto;
import com.ym.mec.biz.dal.entity.Goods;
import com.ym.mec.biz.dal.entity.MusicGroup;
import com.ym.mec.biz.dal.entity.MusicGroupPurchaseList;
import com.ym.mec.biz.dal.entity.Organization;
import com.ym.mec.biz.dal.entity.StudentPaymentOrder;
import com.ym.mec.biz.dal.entity.StudentPaymentOrderDetail;
import com.ym.mec.biz.dal.entity.StudentRegistration;
import com.ym.mec.biz.dal.entity.SubjectChange;
import com.ym.mec.biz.dal.enums.DealStatusEnum;
import com.ym.mec.biz.dal.enums.KitGroupPurchaseTypeEnum;
import com.ym.mec.biz.dal.enums.OrderDetailTypeEnum;
import com.ym.mec.biz.dal.enums.StudentMusicGroupStatusEnum;
import com.ym.mec.biz.service.GoodsService;
import com.ym.mec.biz.service.MusicGroupPaymentCalenderService;
import com.ym.mec.biz.service.StudentPaymentOrderDetailService;
import com.ym.mec.biz.service.StudentPaymentOrderService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.util.collection.MapUtil;

@Service
public class StudentPaymentOrderDetailServiceImpl extends BaseServiceImpl<Long, StudentPaymentOrderDetail> implements StudentPaymentOrderDetailService {

    @Autowired
    private StudentPaymentOrderDetailDao studentPaymentOrderDetailDao;
    @Autowired
    private GoodsService goodsService;
    @Autowired
    private TeacherDao teacherDao;
    @Autowired
    private MusicGroupDao musicGroupDao;
    @Autowired
    private OrganizationDao organizationDao;
    @Autowired
    private MusicGroupPurchaseListDao musicGroupPurchaseListDao;
    @Autowired
    private StudentPaymentOrderDao studentPaymentOrderDao;
    @Autowired
    private StudentRegistrationDao studentRegistrationDao;
    @Autowired
    private SubjectDao subjectDao;
    @Autowired
    private SubjectChangeDao subjectChangeDao;
    @Autowired
    private MusicGroupPaymentCalenderService musicGroupPaymentCalenderService;
    @Autowired
    private MusicGroupPaymentCalenderCourseSettingsDao musicGroupPaymentCalenderCourseSettingsDao;

    @Autowired
    private StudentPaymentOrderService studentPaymentOrderService;

    @Override
    public BaseDAO<Long, StudentPaymentOrderDetail> getDAO() {
        return studentPaymentOrderDetailDao;
    }

    @Override
    public int batchAdd(List<StudentPaymentOrderDetail> studentPaymentOrderDetailList) {
        if(studentPaymentOrderDetailList == null || studentPaymentOrderDetailList.size() == 0){
            return 0;
        }
        return studentPaymentOrderDetailDao.batchAdd(studentPaymentOrderDetailList);
    }

    @Override
    public List<StudentPaymentOrderDetail> findApplyOrderSuccess(String musicGroupId, DealStatusEnum status) {
        return studentPaymentOrderDetailDao.findApplyOrderSuccess(musicGroupId, status);
    }

    @Override
    public List<Goods> getMusicalList(String musicGroupId) {
        List<StudentPaymentOrderDetail> applyOrder = findApplyOrderSuccess(musicGroupId, DealStatusEnum.SUCCESS);
        if (applyOrder == null || applyOrder.size() == 0) {
            return null;
        }
        StringBuilder goodsIdsStr = new StringBuilder();
        for (StudentPaymentOrderDetail studentPaymentOrderDetail : applyOrder) {
            if (studentPaymentOrderDetail == null) continue;
            if (StringUtils.isNotBlank(studentPaymentOrderDetail.getGoodsIdList())) {
                goodsIdsStr.append(studentPaymentOrderDetail.getGoodsIdList()).append(",");
            }
        }
        if (StringUtils.isBlank(goodsIdsStr.toString())) {
            return null;
        }

        String[] goodSIdArr = goodsIdsStr.toString().split(",");

        List<Goods> goodies = goodsService.findGoodsByIds(goodsIdsStr.toString());
        MusicGroup musicGroup = musicGroupDao.get(musicGroupId);
        Organization organization = organizationDao.get(musicGroup.getOrganId());
        goodies.forEach(e -> {
            e.setSellCount(0);
            //当作乐团名称字段处理
            e.setMemo(musicGroup.getName());
            //当作分部名称字段处理
            e.setBrief(organization.getName());
            for (String goodsIdStr : goodSIdArr) {
                if (e.getId().equals(Integer.parseInt(goodsIdStr))) {
                    e.setSellCount(e.getSellCount() + 1);
                }
            }
        });
        //乐器更换的
        SubjectChange changeGoods = subjectChangeDao.getChangeGoods(musicGroupId);
        if (changeGoods == null) {
            return goodies;
        }
        String originalGoodsIds = "";
        String changeGoodsIds = "";
        if (StringUtils.isNotBlank(changeGoods.getMusicGroupName())) {
            originalGoodsIds += StringUtils.isNotBlank(originalGoodsIds) ? "," + changeGoods.getMusicGroupName() : changeGoods.getMusicGroupName();
        }
        if (StringUtils.isNotBlank(changeGoods.getOriginalAccessories())) {
            originalGoodsIds += StringUtils.isNotBlank(originalGoodsIds) ? "," + changeGoods.getOriginalAccessories() : changeGoods.getOriginalAccessories();
        }
        if (StringUtils.isNotBlank(changeGoods.getTransNo())) {
            changeGoodsIds += StringUtils.isNotBlank(changeGoodsIds) ? "," + changeGoods.getTransNo() : changeGoods.getTransNo();
        }
        if (StringUtils.isNotBlank(changeGoods.getChangeAccessories())) {
            changeGoodsIds += StringUtils.isNotBlank(changeGoodsIds) ? "," + changeGoods.getChangeAccessories() : changeGoods.getChangeAccessories();
        }
        if (StringUtils.isNotBlank(changeGoodsIds)) {
            String[] changGoodsIdsArr = changeGoodsIds.split(",");
            List<Goods> addGoodies = new ArrayList<>();
            for (String goodsId : changGoodsIdsArr) {
                boolean has = false;
                for (Goods goods : goodies) {
                    if (goodsId.equals(goods.getId().toString())) {
                        goods.setSellCount(goods.getSellCount() + 1);
                        has = true;
                        break;
                    }
                }

                for (Goods addGoods : addGoodies) {
                    if (goodsId.equals(addGoods.getId().toString())) {
                        addGoods.setSellCount(addGoods.getSellCount() + 1);
                        has = true;
                        break;
                    }
                }
                if (!has) {
                    Goods goods = goodsService.get(Integer.parseInt(goodsId));
                    goods.setSellCount(1);
                    goods.setMemo(musicGroup.getName());
                    //当作分部名称字段处理
                    goods.setBrief(organization.getName());
                    addGoodies.add(goods);
                }
            }
            if (addGoodies.size() > 0) {
                goodies.addAll(addGoodies);
            }
        }

        if (StringUtils.isNotBlank(originalGoodsIds)) {
            String[] originalGoodsIdsArr = originalGoodsIds.split(",");
            for (String goodsId : originalGoodsIdsArr) {
                for (Goods goods : goodies) {
                    if (goodsId.equals(goods.getId().toString())) {
                        goods.setSellCount(goods.getSellCount() - 1);
                        break;
                    }
                }
            }
        }
        goodies.removeIf(goods -> goods.getSellCount() <= 0);
        return goodies;
    }

    @Override
    public Boolean verifyMusicalList(String musicGroupId) throws BizException {
        //乐器采购清单是否确认
        Map<String, Object> param = new HashMap<>();
        param.put("musicGroupId", musicGroupId);
        int count = musicGroupPurchaseListDao.queryCount(param);
        if (count > 0) {
            throw new BizException("采购清单已经确认");
        }

        List<Goods> musicalList = getMusicalList(musicGroupId);
        Date date = new Date();
        List<MusicGroupPurchaseList> musicGroupPurchaseLists = new ArrayList<>();
        //发放清单没有时确认特殊处理
        if (musicalList == null || musicalList.size() == 0) {
            musicalList = new ArrayList<>();
            Goods goods = new Goods();
            goods.setId(0);
            goods.setSellCount(0);
            musicalList.add(goods);
        }

        for (Goods goods : musicalList) {
            MusicGroupPurchaseList musicGroupPurchaseList = new MusicGroupPurchaseList();
            musicGroupPurchaseList.setGoodsId(goods.getId());
            musicGroupPurchaseList.setMusicGroupId(musicGroupId);
            musicGroupPurchaseList.setPurchaseNum(goods.getSellCount());
            musicGroupPurchaseList.setCreateTime(date);
            musicGroupPurchaseList.setUpdateTime(date);
            musicGroupPurchaseLists.add(musicGroupPurchaseList);
        }
        musicGroupPurchaseListDao.batchInsert(musicGroupPurchaseLists);
        return true;
    }

    @Override
    public List<Goods> findApplyOrderGoods(Long orderId) {
        List<StudentPaymentOrderDetail> applyOrderGoods = studentPaymentOrderDetailDao.findApplyOrderGoods(orderId);

        if (applyOrderGoods == null || applyOrderGoods.size() == 0) {
            return null;
        }
        String goodsIdsStr = "";
        for (StudentPaymentOrderDetail studentPaymentOrderDetail : applyOrderGoods) {
            if (studentPaymentOrderDetail.getGoodsIdList() != null && !studentPaymentOrderDetail.getGoodsIdList().isEmpty()) {
                goodsIdsStr += studentPaymentOrderDetail.getGoodsIdList() + ",";
            }
        }
        if (goodsIdsStr.isEmpty()) {
            return null;
        }
        goodsIdsStr = goodsIdsStr.substring(0, goodsIdsStr.length() - 1);

        String[] goodSIdArr = goodsIdsStr.split(",");

        List<Goods> goodies = goodsService.findGoodsByIds(goodsIdsStr);

        for (Goods goods : goodies) {
            goods.setSellCount(0);
            for (String goodsIdStr : goodSIdArr) {
                if (goods.getId().equals(Integer.parseInt(goodsIdStr))) {
                    goods.setSellCount(goods.getSellCount() + 1);
                }
            }
            if (StringUtils.isBlank(goods.getComplementGoodsIdList())) continue;
            goods.setGoodsList(goodsService.findGoodsByIds(goods.getComplementGoodsIdList()));
        }
        return goodies;
    }

    @Override
    public List<StudentPaymentOrderDetail> findUserApplyOrder(Integer userId, String musicGroupId, DealStatusEnum status) {
        return studentPaymentOrderDetailDao.findUserApplyOrder(userId, musicGroupId, status);
    }

    @Override
    public List<MusicalListDetailDto> getMusicalListDetail(String musicGroupId, Integer studentId) {

        List<MusicalListDetailDto> musicalListDetailDtos = new ArrayList<MusicalListDetailDto>();
        
    	List<String> musicGroupIdList = new ArrayList<String>();
    	musicGroupIdList.add(musicGroupId);
    	
    	List<StudentRegistration> studentRegistrationList = studentRegistrationDao.findByMusicGroupIds(musicGroupIdList,StudentMusicGroupStatusEnum.NORMAL);
    	
    	if(studentRegistrationList == null || studentRegistrationList.size() == 0){
    		return musicalListDetailDtos;
    	}

        List<Integer> userIds = studentRegistrationList.stream().distinct().map(StudentRegistration::getUserId).collect(Collectors.toList());
        
        List<SubjectChange> subjectChangeList = subjectChangeDao.getStudentGoods(musicGroupId);
        Map<Integer,SubjectChange> subjectChangeMap = subjectChangeList.stream().collect(Collectors.toMap(SubjectChange :: getStudentId, t -> t));
        
        SubjectChange subjectChange = null;
        
        MusicGroup musicGroup = musicGroupDao.get(musicGroupId);
        Organization organization = organizationDao.get(musicGroup.getOrganId());
        Map<Integer, String> usernameMap = MapUtil.convertIntegerMap(teacherDao.queryUsernameByIds(userIds));
        Map<Integer, String> userSubjectMap = subjectDao.getRegisterSubject(musicGroupId, userIds).stream().collect(Collectors.toMap(StudentRegistration::getUserId, StudentRegistration::getSubjectName));
        Integer userId = null;
        
        for(StudentRegistration studentRegistration : studentRegistrationList){
        	
        	MusicalListDetailDto dto = new MusicalListDetailDto();
        	
        	userId = studentRegistration.getUserId();
        	
        	List<StudentPaymentOrder> studentPaymentOrderList = new ArrayList<StudentPaymentOrder>();

			if (studentRegistration.getMusicGroupPaymentCalenderId() == null) {
				StudentPaymentOrder studentPaymentOrder = studentPaymentOrderService.findMusicGroupApplyOrderByStatus(userId, musicGroupId, SUCCESS);
				if (studentPaymentOrder != null) {
					studentPaymentOrderList.add(studentPaymentOrder);
				}

			} else {
				studentPaymentOrderList = studentPaymentOrderService.queryByBatchNo(userId, studentRegistration.getMusicGroupPaymentCalenderId() + "",
						DealStatusEnum.SUCCESS);
			}

			List<Long> paymentOrderIdList = studentPaymentOrderList.stream().map(t -> t.getId()).collect(Collectors.toList());
			
			Long minPaymentOrderId = Collections.min(paymentOrderIdList);

			List<StudentPaymentOrderDetail> orderDetailList = studentPaymentOrderDetailDao.getWithIds(paymentOrderIdList);
			
			Map<OrderDetailTypeEnum,BigDecimal> userOrderTypeMap = new HashMap<OrderDetailTypeEnum, BigDecimal>();;
			
			BigDecimal amount = BigDecimal.ZERO;
			KitGroupPurchaseTypeEnum kitGroupPurchaseType = null;
			String goodsId = null ,accessoriesId = null ;
			BigDecimal orderTotalAmount = BigDecimal.ZERO;
			BigDecimal withoutMusicalTotalAmount = BigDecimal.ZERO;
			
			for(StudentPaymentOrderDetail spod : orderDetailList){
				
				if (spod.getType() == OrderDetailTypeEnum.COURSE || spod.getType() == OrderDetailTypeEnum.HIGH_ONLINE_COURSE
						|| spod.getType() == OrderDetailTypeEnum.SINGLE || spod.getType() == OrderDetailTypeEnum.VIP
						|| spod.getType() == OrderDetailTypeEnum.HIGH || spod.getType() == OrderDetailTypeEnum.MIX
						|| spod.getType() == OrderDetailTypeEnum.DEMO || spod.getType() == OrderDetailTypeEnum.COMPREHENSIVE
						|| spod.getType() == OrderDetailTypeEnum.ENLIGHTENMENT || spod.getType() == OrderDetailTypeEnum.TRAINING_SINGLE
						|| spod.getType() == OrderDetailTypeEnum.TRAINING_MIX || spod.getType() == OrderDetailTypeEnum.HIGH_ONLINE
						|| spod.getType() == OrderDetailTypeEnum.MUSIC_NETWORK || spod.getType() == OrderDetailTypeEnum.CLASSROOM || spod.getType() == OrderDetailTypeEnum.THEORY_COURSE) {
					spod.setType(OrderDetailTypeEnum.COURSE);
					withoutMusicalTotalAmount = withoutMusicalTotalAmount.add(spod.getPrice().subtract(spod.getRemitFee()));
				}else if(spod.getType() == OrderDetailTypeEnum.MUSICAL){
					kitGroupPurchaseType = spod.getKitGroupPurchaseType();
					goodsId = spod.getGoodsIdList();
				}else if(spod.getType() == OrderDetailTypeEnum.ACCESSORIES){
					accessoriesId = spod.getGoodsIdList();
				}else{
					withoutMusicalTotalAmount = withoutMusicalTotalAmount.add(spod.getPrice().subtract(spod.getRemitFee()));
				}
				
				amount = userOrderTypeMap.get(spod.getType());
				if(amount != null){
					amount = amount.add(spod.getPrice().subtract(spod.getRemitFee()));
				}else{
					amount = spod.getPrice().subtract(spod.getRemitFee());
				}
				userOrderTypeMap.put(spod.getType(), amount);
				
				orderTotalAmount = orderTotalAmount.add(spod.getPrice().subtract(spod.getRemitFee()));
			}
        	
        	subjectChange = subjectChangeMap.get(studentRegistration.getUserId());
        	if(subjectChange != null && minPaymentOrderId <= subjectChange.getOriginalOrderId()){
        		dto.setAccessoriesAmount(subjectChange.getChangeAccessoriesPrice());
        		if(subjectChange.getChangeAccessoriesGoods() != null){
        			dto.setAccessoriesName(subjectChange.getChangeAccessoriesGoods().stream().map(Goods::getName).collect(Collectors.joining(",")));
        		}
        		
        		dto.setCourseAmount(userOrderTypeMap.get(OrderDetailTypeEnum.COURSE));
        		//dto.setGoodsNames(goodsNames);
        		dto.setKitGroupPurchaseTypeEnum(subjectChange.getKitGroupPurchaseType());
        		dto.setMusicalAmount(subjectChange.getChangeMusicalPrice());
        		if(subjectChange.getChangeMusicalGoods() != null){
        			dto.setMusicalName(subjectChange.getChangeMusicalGoods().getName());
        		}
        		dto.setMusicGroupName(musicGroup.getName());
        		dto.setOrganName(organization.getName());
        		dto.setOrderAmount(withoutMusicalTotalAmount.add(subjectChange.getChangeMusicalPrice().add(subjectChange.getChangeAccessoriesPrice())));
        		dto.setUserId(userId);
        		dto.setSubjectName(userSubjectMap.get(userId));
        		dto.setUsername(usernameMap.get(userId));
        	}else{
        		
        		if(StringUtils.isNoneBlank(accessoriesId)){
        			dto.setAccessoriesName(goodsService.findGoodsByIds(accessoriesId).stream().map(Goods::getName).collect(Collectors.joining(",")));
        		}

        		dto.setAccessoriesAmount(userOrderTypeMap.get(OrderDetailTypeEnum.ACCESSORIES));
    			dto.setCourseAmount(userOrderTypeMap.get(OrderDetailTypeEnum.COURSE));
        		dto.setMusicalAmount(userOrderTypeMap.get(OrderDetailTypeEnum.MUSICAL));
    		
        		//dto.setGoodsNames(goodsNames);
        		dto.setKitGroupPurchaseTypeEnum(kitGroupPurchaseType);
        		if(StringUtils.isNoneBlank(goodsId)){
        			dto.setMusicalName(goodsService.findGoodsByIds(goodsId).stream().map(Goods::getName).collect(Collectors.joining(",")));
        		}
        		dto.setMusicGroupName(musicGroup.getName());
        		dto.setOrganName(organization.getName());
        		dto.setOrderAmount(orderTotalAmount);
        		dto.setUserId(userId);
        		dto.setSubjectName(userSubjectMap.get(userId));
        		dto.setUsername(usernameMap.get(userId));
        	}
        	musicalListDetailDtos.add(dto);
        }
    	
        return musicalListDetailDtos;
    }

    @Override
    public StudentPaymentOrderDetail findApplyOrderMusical(Long orderId) {
        return studentPaymentOrderDetailDao.findApplyOrderMusical(orderId);
    }

    @Override
	public List<StudentPaymentOrderDetail> getOrderDetail(Long orderId) {
    	
    	if(orderId == null){
    		return new ArrayList<StudentPaymentOrderDetail>();
    	}
		
    	List<Long> orderIdList = new ArrayList<Long>();
    	orderIdList.add(orderId);
    		
    	return getOrderDetail(orderIdList);
	}

    @Override
	public List<StudentPaymentOrderDetail> getOrderDetail(List<Long> orderIdList) {
    	List<StudentPaymentOrderDetail> studentPaymentOrderDetailList = studentPaymentOrderDetailDao.getOrderDetailByOrderId(orderIdList);

		for (StudentPaymentOrderDetail spod : studentPaymentOrderDetailList) {
			if (spod.getType() == OrderDetailTypeEnum.ACCESSORIES || spod.getType() == OrderDetailTypeEnum.MUSICAL
					|| spod.getType() == OrderDetailTypeEnum.TEACHING) {
				List<Goods> goods = goodsService.findGoodsByIds(spod.getGoodsIdList());
				if (goods != null && goods.size() > 0) {
					spod.setName(goods.stream().map(t -> t.getName()).collect(Collectors.joining(",")));
				}
			}
		}

		return studentPaymentOrderDetailList;
	}

	@Override
    public List<String> getOrderDetailType(Long orderId) {
        return studentPaymentOrderDetailDao.getOrderDetailType(orderId);
    }

    @Override
    public List<StudentPaymentOrderDetail> getOrderGoodsDetail(Long orderId) {
        return studentPaymentOrderDetailDao.getOrderDetail(orderId);
    }
}