package com.ym.mec.biz.service.impl;

import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.*;
import com.ym.mec.biz.dal.page.OrganCloudStudyStudentDataQueryInfo;
import com.ym.mec.biz.dal.page.StudentQueryInfo;
import com.ym.mec.biz.service.StudentService;
import com.ym.mec.biz.service.SysConfigService;
import com.ym.mec.biz.service.SysMessageService;
import com.ym.mec.common.constant.CommonConstants;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.QueryInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import sun.nio.cs.ext.Big5;

import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

@Service
public class StudentServiceImpl extends BaseServiceImpl<Integer, Student> implements StudentService {

    @Autowired
    private StudentDao studentDao;
    @Autowired
    private TeacherDao teacherDao;

    @Autowired
    private CourseScheduleStudentPaymentDao courseScheduleStudentPaymentDao;

    @Autowired
    private StudentRegistrationDao studentRegistrationDao;

    @Autowired
    private MusicGroupDao musicGroupDao;

    @Autowired
    private PracticeGroupDao practiceGroupDao;

    @Autowired
    private MemberRankSettingDao memberRankSettingDao;

    @Autowired
    private SysConfigDao sysConfigDao;
    @Autowired
    private SysMessageService sysMessageService;

    @Autowired
    private SysMusicCompareRecordDao sysMusicCompareRecordDao;

    @Autowired
    private CloudTeacherDao cloudTeacherDao;

    @Autowired
    private OrganizationDao organizationDao;

    @Override
    public BaseDAO<Integer, Student> getDAO() {
        return studentDao;
    }

    @Override
    public PageInfo findStudentVipGroupList(StudentQueryInfo queryInfo) {
        PageInfo pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);

        List dataList = null;
        int count = studentDao.countStudents(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = studentDao.findStudents(params);
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long upSet(Student student) {
        if (studentDao.get(student.getUserId()) == null) {
            return studentDao.insert(student);
        } else {
            student.setUpdateTime(new Date());
            studentDao.update(student);
            return student.getUserId().longValue();
        }
    }

    @Override
    public boolean updateOperatingTempTag() {
        List<StudentCourseTimesDto> studentCourseTimesDtoList = courseScheduleStudentPaymentDao.queryStudentNotStartCourseTimesOfOnline();
        Map<Integer, StudentCourseTimesDto> map = studentCourseTimesDtoList.stream().collect(Collectors.toMap(StudentCourseTimesDto::getUserId, s -> s));
        //查询服务指标为0的用户
        List<Student> unlabeledStudentList = studentDao.queryByOperatingTempTag(0);

        List<Student> updateStudentList = new ArrayList<Student>();
        StudentCourseTimesDto dto = null;
        for (Student s : unlabeledStudentList) {
            if (s.getOperatingTag() == 1) {
                continue;
            }
            dto = map.get(s.getUserId());
            if (dto != null) {
                if (dto.getTotalCourseTimes() > 0 && dto.getTotalCourseTimes() != dto.getFreePracticeCourseTimes()) {
                    s.setOperatingTempTag(1);
                    s.setOperatingTag(1);
                    updateStudentList.add(s);
                }
            }
        }

        List<Student> labeledStudentList = studentDao.queryByOperatingTempTag(1);
        for (Student s : labeledStudentList) {
            dto = map.get(s.getUserId());
            if (dto == null || dto.getTotalCourseTimes() == dto.getFreePracticeCourseTimes()) {
                s.setOperatingTempTag(0);
                s.setOperatingTag(0);
                updateStudentList.add(s);
            }
        }

        if (updateStudentList.size() > 0) {
            studentDao.batchUpdate(updateStudentList);
        }

        return true;
    }

    @Override
    public void initTeacherId() {
        List<StudentTeacherCourseDto> allStudentCourseInfo = courseScheduleStudentPaymentDao.findAllStudentCourseInfo();
        Map<Integer, List<StudentTeacherCourseDto>> studentCoursesMap = allStudentCourseInfo.stream().collect(Collectors.groupingBy(StudentTeacherCourseDto::getStudentId));
        List<Student> students = new ArrayList<>();
        for (Map.Entry<Integer, List<StudentTeacherCourseDto>> studentCoursesEntry : studentCoursesMap.entrySet()) {
            Map<CourseSchedule.CourseScheduleType, List<StudentTeacherCourseDto>> courseTypeCourseMap = studentCoursesEntry.getValue().stream().collect(Collectors.groupingBy(StudentTeacherCourseDto::getCourseScheduleType));
            List<StudentTeacherCourseDto> practiceCourses = courseTypeCourseMap.get(CourseSchedule.CourseScheduleType.PRACTICE);
            if (!CollectionUtils.isEmpty(practiceCourses)) {
                practiceCourses.sort(Comparator.comparing(StudentTeacherCourseDto::getClassesStartTime).reversed());
                Student student = new Student(studentCoursesEntry.getKey());
                student.setTeacherId(practiceCourses.get(0).getTeacherId());
                students.add(student);
                continue;
            }

            List<StudentTeacherCourseDto> vipCourses = courseTypeCourseMap.get(CourseSchedule.CourseScheduleType.VIP);
            if (!CollectionUtils.isEmpty(vipCourses)) {
                Student student = new Student(studentCoursesEntry.getKey());
                vipCourses.sort(Comparator.comparing(StudentTeacherCourseDto::getClassesStartTime).reversed());
                student.setTeacherId(vipCourses.get(0).getTeacherId());
                students.add(student);
                continue;
            }

            List<StudentTeacherCourseDto> singleCourses = courseTypeCourseMap.get(CourseSchedule.CourseScheduleType.SINGLE);
            if (!CollectionUtils.isEmpty(singleCourses)) {
                Student student = new Student(studentCoursesEntry.getKey());
                singleCourses.sort(Comparator.comparing(StudentTeacherCourseDto::getClassesStartTime).reversed());
                student.setTeacherId(singleCourses.get(0).getTeacherId());
                students.add(student);
                continue;
            }

            List<StudentTeacherCourseDto> mixCourses = courseTypeCourseMap.get(CourseSchedule.CourseScheduleType.MIX);
            if (!CollectionUtils.isEmpty(mixCourses)) {
                Student student = new Student(studentCoursesEntry.getKey());
                mixCourses.sort(Comparator.comparing(StudentTeacherCourseDto::getClassesStartTime).reversed());
                student.setTeacherId(mixCourses.get(0).getTeacherId());
                students.add(student);
                continue;
            }
        }
        if (!CollectionUtils.isEmpty(students)) {
            studentDao.batchUpdate(students);
        }
        students = null;
        studentCoursesMap = null;
        allStudentCourseInfo = null;
    }

    @Override
    public void batchUpdateAdviser(Integer teacherId, String studentIds) {
        studentDao.batchUpdateAdviser(teacherId, studentIds);
    }

    @Override
    public Integer updateGrade() {
        return studentDao.updateGrade();
    }

    @Override
    public String getStudentGrade(GradeTypeEnum gradeType, Integer gradeNum) {
        String grade = "";
        if (GradeTypeEnum.FIVE_PLUS.equals(gradeType)) {
            FivePlusGradeEnum[] fivePlusGradeEnums = FivePlusGradeEnum.values();
            for (FivePlusGradeEnum fivePlusGradeEnum : fivePlusGradeEnums) {
                if (!fivePlusGradeEnum.getCode().equals(gradeNum)) continue;
                grade = fivePlusGradeEnum.getDesc();
            }
        }
        if (GradeTypeEnum.SIX_PLUS.equals(gradeType)) {
            SixPlusGradeEnum[] sixPlusGradeEnums = SixPlusGradeEnum.values();
            for (SixPlusGradeEnum sixPlusGradeEnum : sixPlusGradeEnums) {
                if (!sixPlusGradeEnum.getCode().equals(gradeNum)) continue;
                grade = sixPlusGradeEnum.getDesc();
            }
        }
        return grade;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateStudentTeacherAndSubject() {
        //更新新学员状态
        List<Integer> newStudentIds = studentDao.getHasVipCourseStudentIdsFromNewStudents();
        if(!CollectionUtils.isEmpty(newStudentIds)){
            studentDao.updateNewStudentsToOld(newStudentIds);
        }

        List<Student> noTeacherOrNoSubjectStudent = studentDao.getNoTeacherOrNoSubjectStudent();
        if (CollectionUtils.isEmpty(noTeacherOrNoSubjectStudent)) {
            return;
        }
        List<Integer> studentIds = noTeacherOrNoSubjectStudent.stream().map(Student::getUserId).collect(Collectors.toList());
        List<StudentClassInfoDto> studentClassInfo = studentRegistrationDao.getStudentClassInfo(studentIds);
        if (CollectionUtils.isEmpty(studentClassInfo)) {
            return;
        }
        //更新学员指导老师和声部信息
        List<Student> needUpdateStudents = new ArrayList<>();
        Map<Integer, List<StudentClassInfoDto>> scm = studentClassInfo.stream().collect(Collectors.groupingBy(StudentClassInfoDto::getStudentId));
        for (Student student : noTeacherOrNoSubjectStudent) {
            if (!scm.containsKey(student.getUserId())) {
                continue;
            }
            List<StudentClassInfoDto> studentClassInfoDtos = scm.get(student.getUserId());
            StudentClassInfoDto studentClassInfoDto = studentClassInfoDtos.stream().max(Comparator.comparing(StudentClassInfoDto::getClassGroupId)).get();
            if (Objects.isNull(student.getTeacherId())) {
                student.setTeacherId(studentClassInfoDto.getMainTeacherId());
            }
            if (StringUtils.isBlank(student.getSubjectIdList())) {
                student.setSubjectIdList(studentClassInfoDto.getSubjectId().toString());
            }
            if (Objects.isNull(student.getCooperationOrganId())) {
                student.setCooperationOrganId(studentClassInfoDto.getCooperationOrganId());
            }
            needUpdateStudents.add(student);
        }
        if (!CollectionUtils.isEmpty(needUpdateStudents)) {
            studentDao.batchUpdate(needUpdateStudents);
        }
    }

    @Override
    public CooperationOrgan getStudentEduTeacher(Integer userId) {
        CooperationOrgan cooperationOrgan = musicGroupDao.findUserMusicGroupCooperationOrgan(userId);
        if (cooperationOrgan != null) {
            return cooperationOrgan;
        }
        cooperationOrgan = new CooperationOrgan();
        String studentEduTeacher = courseScheduleStudentPaymentDao.getStudentEduTeacher(userId);
        if (studentEduTeacher == null) {
            studentEduTeacher = practiceGroupDao.getStudentEduTeacher(userId);
        }
        cooperationOrgan.setLinkman(studentEduTeacher);
        return cooperationOrgan;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateMemberRank(CloudTeacherOrder cloudTeacherOrder, PeriodEnum periodEnum) {
        // 添加会员有效时长
        Student student = this.get(cloudTeacherOrder.getStudentId());
        if (student == null) {
            throw new BizException("学员信息不存在");
        }

        Date nowDate = new Date();
        if (student.getMemberRankSettingId() == null || nowDate.after(student.getMembershipEndTime())) {
        	switch (periodEnum) {
    		case DAY:
                student.setMembershipEndTime(DateUtil.addDays(nowDate, cloudTeacherOrder.getTime()));
    			break;
    		case MONTH:
    		case YEAR_HALF:
                student.setMembershipEndTime(DateUtil.addMonths(nowDate, cloudTeacherOrder.getTime()));
    			break;
    		case YEAR:
                student.setMembershipEndTime(DateUtil.addYears(nowDate, cloudTeacherOrder.getTime()));
    			break;

    		default:
    			break;
    		}
            student.setMembershipStartTime(nowDate);
        } else {
        	switch (periodEnum) {
    		case DAY:
            	student.setMembershipEndTime(DateUtil.addDays(student.getMembershipEndTime(), cloudTeacherOrder.getTime()));
    			break;
    		case MONTH:
    		case YEAR_HALF:
            	student.setMembershipEndTime(DateUtil.addMonths(student.getMembershipEndTime(), cloudTeacherOrder.getTime()));
    			break;
    		case YEAR:
            	student.setMembershipEndTime(DateUtil.addYears(student.getMembershipEndTime(), cloudTeacherOrder.getTime()));
    			break;
    		default:
    			break;
    		}
        }
        if(StringUtils.isNotEmpty(cloudTeacherOrder.getMusicGroupId())){
            StudentRegistration registration = studentRegistrationDao.getByUserIdAndMusicGroupId(cloudTeacherOrder.getMusicGroupId(), cloudTeacherOrder.getStudentId());
            if(registration != null && (registration.getMusicGroupStatus() == StudentMusicGroupStatusEnum.NORMAL || registration.getMusicGroupStatus() == StudentMusicGroupStatusEnum.APPLY)){
                if (registration.getMembershipEndTime() == null || nowDate.after(registration.getMembershipEndTime())) {
                    switch (periodEnum) {
                        case DAY:
                            registration.setMembershipEndTime(DateUtil.addDays(nowDate, cloudTeacherOrder.getTime()));
                            break;
                        case MONTH:
                        case YEAR_HALF:
                            registration.setMembershipEndTime(DateUtil.addMonths(nowDate, cloudTeacherOrder.getTime()));
                            break;
                        case YEAR:
                            registration.setMembershipEndTime(DateUtil.addYears(nowDate, cloudTeacherOrder.getTime()));
                            break;
                        default:
                            break;
                    }
                } else {
                    switch (periodEnum) {
                        case DAY:
                            registration.setMembershipEndTime(DateUtil.addDays(registration.getMembershipEndTime(), cloudTeacherOrder.getTime()));
                            break;
                        case MONTH:
                        case YEAR_HALF:
                            registration.setMembershipEndTime(DateUtil.addMonths(registration.getMembershipEndTime(), cloudTeacherOrder.getTime()));
                            break;
                        case YEAR:
                            registration.setMembershipEndTime(DateUtil.addYears(registration.getMembershipEndTime(), cloudTeacherOrder.getTime()));
                            break;
                        default:
                            break;
                    }
                }
                studentRegistrationDao.update(registration);
            }
        }

        student.setMemberRankSettingId(cloudTeacherOrder.getLevel());
        student.setUpdateTime(nowDate);
        this.update(student);
    }

    @Override
    public void updateStudentServiceTag(Integer studentId,List<Integer> studentIds,Integer serviceTag) {
        studentDao.updateStudentServiceTag(studentId, studentIds, serviceTag);
    }

    @Override
    public List<String> getStudentNames(List<Integer> studentIdList) {
        return studentDao.getStudentNames(studentIdList);
    }

	@Override
	public Student getLocked(Integer userId) {
		return studentDao.getLocked(userId);
	}

    @Override
    public void cleanMember(Integer userId) {
        studentDao.cleanMember(userId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateMemberExperience(String studentIds, Integer rankId, Date endTime) {
        //会员是否存在
        MemberRankSetting memberRankSetting = memberRankSettingDao.get(rankId);
        if(memberRankSetting == null){
            throw new BizException("会员信息不存在");
        }
        if(StringUtils.isEmpty(studentIds)){
            throw new BizException("请选择学员");
        }
        //学员是否有会员
        List<Integer> studentIdList = Arrays.stream(studentIds.split(",")).map(Integer::parseInt).collect(Collectors.toList());
        List<Student> studentList = studentDao.findByStudentIds(studentIdList);
        List<Student> collect = studentList.stream().filter(e -> e.getMemberRankSettingId() != null).collect(Collectors.toList());
        if(collect.size() > 0){
            StringBuffer sb = new StringBuffer("操作失败：学员 ");
            List<Integer> studentId = collect.stream().map(e -> e.getUserId()).collect(Collectors.toList());
            Map<Integer, String> maps = MapUtil.convertMybatisMap(teacherDao.queryUserNameByIdList(studentId));
            for (Integer integer : studentId) {
                sb.append(maps.get(integer)).append(" ");
            }
            sb.append("已有会员权限");
            throw new BizException(sb.toString());
        }
        //最多到10月1号
        String configValue = sysConfigDao.findConfigValue(SysConfigService.EXPERIENCE_MEMBERSHIP_END_TIME);
        if(StringUtils.isNotEmpty(configValue)){
            Date date = DateUtil.stringToDate(configValue, DateUtil.ISO_EXPANDED_DATE_FORMAT);
            if(endTime.after(date)){
                throw new BizException("操作失败：会员试用期不可超过{}",configValue);
            }
        }

        int i = studentDao.updateMemberExperience(studentIds, rankId, endTime);

        if(studentIdList.size() == i){
            Map<Integer, String> studentPhoneMap = MapUtil.convertMybatisMap(teacherDao.queryPhoneByIds(studentIds));
            Map<Integer,String> userMap = new HashMap<>(studentIdList.size());
            Map<Integer,String> userMap1 = new HashMap<>(studentIdList.size());
            for (Integer studentId : studentIdList) {
                userMap.put(studentId,studentPhoneMap.get(studentId));
                userMap1.put(studentId,studentId.toString());
            }
            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.AWSMS,
                    MessageTypeEnum.PUSH_EXPERIENCE_MEMBERSHIP_MESSAGE,
                    userMap,
                    null,
                    0,
                    null,
                    null);

            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG,
                    MessageTypeEnum.PUSH_EXPERIENCE_MEMBERSHIP_MESSAGE,
                    userMap1,
                    null,
                    0,
                    null,
                    "STUDENT");
        }
    }

    @Override
    public StatDto organStudentData(StudentQueryInfo queryInfo) {
        PageInfo<EduOrganStudentListDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);

        List<EduOrganStudentListDto> dataList = new ArrayList<>();
        int count = studentDao.countCloudStudyStudentData(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = studentDao.queryCloudStudyStudentData(params);
            List<Integer> studentIds = dataList.stream().map(EduOrganStudentListDto::getStudentId).collect(Collectors.toList());
            List<Map<Integer, String>> studentGroupNamesMapList = studentRegistrationDao.queryStudentMusicGroupNamesMap(studentIds);
            Map<Integer, String> studentGroupNamesMap = MapUtil.convertIntegerMap(studentGroupNamesMapList);
            Set<Integer> hasVipCourseStudentIds = courseScheduleStudentPaymentDao.getHasVipCourseStudentIds(studentIds);

            for (EduOrganStudentListDto eduOrganStudentListDto : dataList) {
                if(studentGroupNamesMap.containsKey(eduOrganStudentListDto.getStudentId())){
                    eduOrganStudentListDto.setMusicGroupNames(studentGroupNamesMap.get(eduOrganStudentListDto.getStudentId()));
                }
                if(hasVipCourseStudentIds.contains(eduOrganStudentListDto.getStudentId())){
                    eduOrganStudentListDto.setHasVipGroup(1);
                }
                if(eduOrganStudentListDto.getCloudStudyUseNum()>0 && eduOrganStudentListDto.getCloudStudyUseTime()>0){
                    eduOrganStudentListDto.setCloudStudyUseAvgTime(eduOrganStudentListDto.getCloudStudyUseTime()/eduOrganStudentListDto.getCloudStudyUseNum());
                }
                eduOrganStudentListDto.setCloudStudyUseTime(eduOrganStudentListDto.getCloudStudyUseTime()/60);
                eduOrganStudentListDto.setCloudStudyUseAvgTime(eduOrganStudentListDto.getCloudStudyUseAvgTime()/60);
            }
        }
        pageInfo.setRows(dataList);

        StatDto result = new StatDto();
        if(queryInfo.getPage()==1){
            EduOrganStudentDataDto organStudentVipData = new EduOrganStudentDataDto();
            organStudentVipData.setTotalStudentNum(studentDao.getOrganStudentNum(queryInfo.getOrganId()));
            organStudentVipData.setVipStudentNum(cloudTeacherDao.getOrganVipStudentNum(queryInfo.getOrganId()));
//            organStudentVipData.seteVipStudentNum(studentDao.getOrganEVipStudentNum(queryInfo.getOrganId()));
            organStudentVipData.setCloudStudyUseStudentNum(studentDao.getCloudStudyStudentNum(queryInfo.getOrganId()));
            organStudentVipData.setCloudStudyLivelyStudentNum(studentDao.getCloudStudyLivelyStudentNum(queryInfo.getOrganId()));
            organStudentVipData.setNewCloudStudyStudentNum(sysMusicCompareRecordDao.getOrganNewCloudStudyNum(queryInfo.getOrganId()));
            if(organStudentVipData.getVipStudentNum()>0&&organStudentVipData.getTotalStudentNum()>0){
                organStudentVipData.setVipStudentDuty(new BigDecimal(organStudentVipData.getVipStudentNum()).divide(new BigDecimal(organStudentVipData.getTotalStudentNum()), 4, BigDecimal.ROUND_DOWN).multiply(new BigDecimal(100)).floatValue());
            }
            result.setHead(organStudentVipData);
        }
        result.setDetail(pageInfo);

        return result;
    }

    @Override
    public List<EduOrganStudentDataDto> organStudentOverView(List<Integer> organIds) {
        List<Organization> organs = organizationDao.getOrgans(organIds);

        if(CollectionUtils.isEmpty(organs)){
            return Collections.emptyList();
        }

        List<Map<Integer, Integer>> organsStudentNumMapList = studentDao.getOrgansStudentNum(organIds);
        Map<String, Long> organsStudentNumMap = new HashMap<>();
        if(!CollectionUtils.isEmpty(organsStudentNumMapList)){
            organsStudentNumMap = MapUtil.convertIntegerMap(organsStudentNumMapList);
        }
        List<Map<Integer, Integer>> organsVipStudentNumMapList = cloudTeacherDao.getOrgansVipStudentNum(organIds);
        Map<String, Long> organsVipStudentNumMap = new HashMap<>();
        if(!CollectionUtils.isEmpty(organsVipStudentNumMapList)){
            organsVipStudentNumMap = MapUtil.convertIntegerMap(organsVipStudentNumMapList);
        }
//        List<Map<Integer, Integer>> organsEVipStudentNumMapList = studentDao.getOrgansEVipStudentNum(organIds);
//        Map<String, Long> organsEVipStudentNumMap = new HashMap<>();
//        if(!CollectionUtils.isEmpty(organsEVipStudentNumMapList)){
//            organsEVipStudentNumMap = MapUtil.convertIntegerMap(organsEVipStudentNumMapList);
//        }
        List<Map<Integer, Integer>> organCloudStudyStudentNumMapList = studentDao.getOrganCloudStudyStudentNum(organIds);
        Map<String, Long> organCloudStudyStudentNumMap = new HashMap<>();
        if(!CollectionUtils.isEmpty(organCloudStudyStudentNumMapList)){
            organCloudStudyStudentNumMap = MapUtil.convertIntegerMap(organCloudStudyStudentNumMapList);
        }
        List<Map<Integer, Integer>> organCloudStudyLivelyStudentNumMapList = studentDao.getOrganCloudStudyLivelyStudentNum(organIds);
        Map<String, Long> organCloudStudyLivelyStudentNumMap = new HashMap<>();
        if(!CollectionUtils.isEmpty(organCloudStudyLivelyStudentNumMapList)){
            organCloudStudyLivelyStudentNumMap = MapUtil.convertIntegerMap(organCloudStudyLivelyStudentNumMapList);
        }
        List<Map<Integer, Integer>> organsNewCloudStudyNumMapList = sysMusicCompareRecordDao.getOrgansNewCloudStudyNum(organIds);
        Map<String, Long> organsNewCloudStudyNumMap = new HashMap<>();
        if(!CollectionUtils.isEmpty(organsNewCloudStudyNumMapList)){
            organsNewCloudStudyNumMap = MapUtil.convertIntegerMap(organsNewCloudStudyNumMapList);
        }
        //获取新增人数占比（已上乐团课小于等于4）
        //获取有在读学员的乐团编号
        //获取有在读学员的学员编号
        List<String> musicGroupIds = studentRegistrationDao.findMusicGroupIds();
        List<Integer> studentIdList = studentRegistrationDao.findStudentIds();
        List<BaseMapDto<Integer, Long>> studentMusicCourseNum = studentDao.getOrganCloudNewStudentNum(musicGroupIds,studentIdList);
        //获取有乐团没排课的数据
        List<Integer> noCourseStudentIdList = studentRegistrationDao.findNoCourseStudentIds();
        List<Integer> userIdList = new ArrayList<>();
        if(!CollectionUtils.isEmpty(studentMusicCourseNum)){
            List<BaseMapDto<Integer, Long>> collect = studentMusicCourseNum.stream().filter(e -> e.getValue() < 5).collect(Collectors.toList());
            userIdList = collect.stream().map(e -> e.getKey()).collect(Collectors.toList());
        }
        userIdList.addAll(noCourseStudentIdList);
        List<Map<Long, Long>> organsNewCloudNewStudentNumMapList = studentDao.groupOrganId(organIds,userIdList);
        Map<String, Long> organsNewCloudNewStudentNumMap = MapUtil.convertIntegerMap(organsNewCloudNewStudentNumMapList);

        List<EduOrganStudentDataDto> result = new ArrayList<>();

        for (Organization organ : organs) {
            EduOrganStudentDataDto organStudentVipData = new EduOrganStudentDataDto();
            organStudentVipData.setOrganId(organ.getId());
            organStudentVipData.setOrganName(organ.getName());
            if(organsStudentNumMap.containsKey(organ.getId().toString())){
                organStudentVipData.setTotalStudentNum(organsStudentNumMap.get(organ.getId().toString()).intValue());
            }
            if(organsVipStudentNumMap.containsKey(organ.getId().toString())){
                organStudentVipData.setVipStudentNum(organsVipStudentNumMap.get(organ.getId().toString()).intValue());
            }
//            if(organsEVipStudentNumMap.containsKey(organ.getId().toString())){
//                organStudentVipData.seteVipStudentNum(organsEVipStudentNumMap.get(organ.getId().toString()).intValue());
//            }
            if(organCloudStudyStudentNumMap.containsKey(organ.getId().toString())){
                organStudentVipData.setCloudStudyUseStudentNum(organCloudStudyStudentNumMap.get(organ.getId().toString()).intValue());
            }
            if(organCloudStudyLivelyStudentNumMap.containsKey(organ.getId().toString())){
                organStudentVipData.setCloudStudyLivelyStudentNum(organCloudStudyLivelyStudentNumMap.get(organ.getId().toString()).intValue());
            }
            if(organStudentVipData.getVipStudentNum()>0&&organStudentVipData.getTotalStudentNum()>0){
                organStudentVipData.setVipStudentDuty(new BigDecimal(organStudentVipData.getVipStudentNum()).divide(new BigDecimal(organStudentVipData.getTotalStudentNum()), 4, BigDecimal.ROUND_DOWN).multiply(new BigDecimal(100)).floatValue());
            }
            if(organStudentVipData.getCloudStudyUseStudentNum()>0&&organStudentVipData.getTotalStudentNum()>0){
                organStudentVipData.setCloudStudyUseStudentDuty(new BigDecimal(organStudentVipData.getCloudStudyUseStudentNum()).divide(new BigDecimal(organStudentVipData.getTotalStudentNum()), 4, BigDecimal.ROUND_DOWN).multiply(new BigDecimal(100)).floatValue());
            }
            if(organsNewCloudStudyNumMap.containsKey(organ.getId().toString())){
                organStudentVipData.setNewCloudStudyStudentNum(organsNewCloudStudyNumMap.get(organ.getId().toString()).intValue());
            }
            if(organsNewCloudNewStudentNumMap.containsKey(organ.getId().toString())){
                organStudentVipData.setNewStudentNum(organsNewCloudNewStudentNumMap.get(organ.getId().toString()).intValue());
                organStudentVipData.setNewStudentDuty(new BigDecimal(organStudentVipData.getNewStudentNum()).divide(new BigDecimal(organStudentVipData.getTotalStudentNum()), 4, BigDecimal.ROUND_DOWN).multiply(new BigDecimal(100)).floatValue());
            }
            result.add(organStudentVipData);
        }
        return result;
    }

    @Override
    public PageInfo<EduOrganStudentDataDto> queryOrganStudentOverView(List<Integer> organIds, OrganCloudStudyStudentDataQueryInfo queryInfo) {
        PageInfo<EduOrganStudentDataDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());

        if(StringUtils.isNotBlank(queryInfo.getOrganIds())&&CollectionUtils.isEmpty(organIds)){
            return pageInfo;
        }

        List<Organization> organs = organizationDao.getOrgans(organIds);
        pageInfo.setTotal(organs.size());

        if(CollectionUtils.isEmpty(organs)){
            return pageInfo;
        }

        List<Map<Integer, Integer>> organsStudentNumMapList = studentDao.getOrgansStudentNum(organIds);
        Map<String, Long> organsStudentNumMap = new HashMap<>();
        if(!CollectionUtils.isEmpty(organsStudentNumMapList)){
            organsStudentNumMap = MapUtil.convertIntegerMap(organsStudentNumMapList);
        }
        List<Map<Integer, Integer>> organsVipStudentNumMapList = cloudTeacherDao.getOrgansVipStudentNum(organIds);
        Map<String, Long> organsVipStudentNumMap = new HashMap<>();
        if(!CollectionUtils.isEmpty(organsVipStudentNumMapList)){
            organsVipStudentNumMap = MapUtil.convertIntegerMap(organsVipStudentNumMapList);
        }
//        List<Map<Integer, Integer>> organsEVipStudentNumMapList = studentDao.getOrgansEVipStudentNum(organIds);
//        Map<String, Long> organsEVipStudentNumMap = new HashMap<>();
//        if(!CollectionUtils.isEmpty(organsEVipStudentNumMapList)){
//            organsEVipStudentNumMap = MapUtil.convertIntegerMap(organsEVipStudentNumMapList);
//        }
        List<Map<Integer, Integer>> organCloudStudyStudentNumMapList = studentDao.getOrganCloudStudyStudentNum(organIds);
        Map<String, Long> organCloudStudyStudentNumMap = new HashMap<>();
        if(!CollectionUtils.isEmpty(organCloudStudyStudentNumMapList)){
            organCloudStudyStudentNumMap = MapUtil.convertIntegerMap(organCloudStudyStudentNumMapList);
        }
        List<Map<Integer, Integer>> organCloudStudyLivelyStudentNumMapList = studentDao.getOrganCloudStudyLivelyStudentNum(organIds);
        Map<String, Long> organCloudStudyLivelyStudentNumMap = new HashMap<>();
        if(!CollectionUtils.isEmpty(organCloudStudyLivelyStudentNumMapList)){
            organCloudStudyLivelyStudentNumMap = MapUtil.convertIntegerMap(organCloudStudyLivelyStudentNumMapList);
        }
        List<Map<Integer, Integer>> organsNewCloudStudyNumMapList = sysMusicCompareRecordDao.getOrgansNewCloudStudyNum(organIds);
        Map<String, Long> organsNewCloudStudyNumMap = new HashMap<>();
        if(!CollectionUtils.isEmpty(organsNewCloudStudyNumMapList)){
            organsNewCloudStudyNumMap = MapUtil.convertIntegerMap(organsNewCloudStudyNumMapList);
        }

        //获取新增人数占比（已上乐团课小于等于4）
        //获取有在读学员的乐团编号
        //获取有在读学员的学员编号
        List<String> musicGroupIds = studentRegistrationDao.findMusicGroupIds();
        List<Integer> studentIdList = studentRegistrationDao.findStudentIds();
        List<BaseMapDto<Integer, Long>> studentMusicCourseNum = studentDao.getOrganCloudNewStudentNum(musicGroupIds,studentIdList);
        //获取有乐团没排课的数据
        List<Integer> noCourseStudentIdList = studentRegistrationDao.findNoCourseStudentIds();
        List<Integer> userIdList = new ArrayList<>();
        if(!CollectionUtils.isEmpty(studentMusicCourseNum)){
            List<BaseMapDto<Integer, Long>> collect = studentMusicCourseNum.stream().filter(e -> e.getValue() < 5).collect(Collectors.toList());
            userIdList = collect.stream().map(e -> e.getKey()).collect(Collectors.toList());
        }
        userIdList.addAll(noCourseStudentIdList);
        List<Map<Long, Long>> organsNewCloudNewStudentNumMapList = studentDao.groupOrganId(organIds,userIdList);
        Map<String, Long> organsNewCloudNewStudentNumMap = MapUtil.convertIntegerMap(organsNewCloudNewStudentNumMapList);

        List<EduOrganStudentDataDto> result = new ArrayList<>();

        for (Organization organ : organs) {
            EduOrganStudentDataDto organStudentVipData = new EduOrganStudentDataDto();
            organStudentVipData.setOrganId(organ.getId());
            organStudentVipData.setOrganName(organ.getName());
            if(organsStudentNumMap.containsKey(organ.getId().toString())){
                organStudentVipData.setTotalStudentNum(organsStudentNumMap.get(organ.getId().toString()).intValue());
            }
            if(organsVipStudentNumMap.containsKey(organ.getId().toString())){
                organStudentVipData.setVipStudentNum(organsVipStudentNumMap.get(organ.getId().toString()).intValue());
            }
//            if(organsEVipStudentNumMap.containsKey(organ.getId().toString())){
//                organStudentVipData.seteVipStudentNum(organsEVipStudentNumMap.get(organ.getId().toString()).intValue());
//            }
            if(organCloudStudyStudentNumMap.containsKey(organ.getId().toString())){
                organStudentVipData.setCloudStudyUseStudentNum(organCloudStudyStudentNumMap.get(organ.getId().toString()).intValue());
            }
            if(organCloudStudyLivelyStudentNumMap.containsKey(organ.getId().toString())){
                organStudentVipData.setCloudStudyLivelyStudentNum(organCloudStudyLivelyStudentNumMap.get(organ.getId().toString()).intValue());
            }
            if(organStudentVipData.getVipStudentNum()>0&&organStudentVipData.getTotalStudentNum()>0){
                organStudentVipData.setVipStudentDuty(new BigDecimal(organStudentVipData.getVipStudentNum()).divide(new BigDecimal(organStudentVipData.getTotalStudentNum()), 4, BigDecimal.ROUND_DOWN).multiply(new BigDecimal(100)).floatValue());
            }
            if(organStudentVipData.getCloudStudyUseStudentNum()>0&&organStudentVipData.getTotalStudentNum()>0){
                organStudentVipData.setCloudStudyUseStudentDuty(new BigDecimal(organStudentVipData.getCloudStudyUseStudentNum()).divide(new BigDecimal(organStudentVipData.getTotalStudentNum()), 4, BigDecimal.ROUND_DOWN).multiply(new BigDecimal(100)).floatValue());
            }
            if(organsNewCloudStudyNumMap.containsKey(organ.getId().toString())){
                organStudentVipData.setNewCloudStudyStudentNum(organsNewCloudStudyNumMap.get(organ.getId().toString()).intValue());
            }
            if(organsNewCloudNewStudentNumMap.containsKey(organ.getId().toString())){
                organStudentVipData.setNewStudentNum(organsNewCloudNewStudentNumMap.get(organ.getId().toString()).intValue());
                organStudentVipData.setNewStudentDuty(new BigDecimal(organStudentVipData.getNewStudentNum()).divide(new BigDecimal(organStudentVipData.getTotalStudentNum()), 4, BigDecimal.ROUND_DOWN).multiply(new BigDecimal(100)).floatValue());
            }
            result.add(organStudentVipData);
        }
        Comparator<EduOrganStudentDataDto> comparing = null;
        if(StringUtils.isNotBlank(queryInfo.getTotalStudentNum())){
            if(Objects.isNull(comparing)){
                comparing = Comparator.comparing(EduOrganStudentDataDto::getTotalStudentNum, "ASC".equals(queryInfo.getTotalStudentNum())?Comparator.naturalOrder():Comparator.reverseOrder());
            }else{
                comparing.thenComparing(EduOrganStudentDataDto::getTotalStudentNum, "ASC".equals(queryInfo.getTotalStudentNum())?Comparator.naturalOrder():Comparator.reverseOrder());
            }
        }
        if(StringUtils.isNotBlank(queryInfo.getCloudStudyLivelyStudentNum())){
            if(Objects.isNull(comparing)){
                comparing = Comparator.comparing(EduOrganStudentDataDto::getCloudStudyLivelyStudentNum, "ASC".equals(queryInfo.getCloudStudyLivelyStudentNum())?Comparator.naturalOrder():Comparator.reverseOrder());
            }else{
                comparing.thenComparing(EduOrganStudentDataDto::getCloudStudyLivelyStudentNum, "ASC".equals(queryInfo.getCloudStudyLivelyStudentNum())?Comparator.naturalOrder():Comparator.reverseOrder());
            }
        }
        if(StringUtils.isNotBlank(queryInfo.getCloudStudyUseStudentDuty())){
            if(Objects.isNull(comparing)){
                comparing = Comparator.comparing(EduOrganStudentDataDto::getCloudStudyUseStudentDuty, "ASC".equals(queryInfo.getCloudStudyUseStudentDuty())?Comparator.naturalOrder():Comparator.reverseOrder());
            }else{
                comparing.thenComparing(EduOrganStudentDataDto::getCloudStudyUseStudentDuty, "ASC".equals(queryInfo.getCloudStudyUseStudentDuty())?Comparator.naturalOrder():Comparator.reverseOrder());
            }
        }
        if(StringUtils.isNotBlank(queryInfo.getVipStudentNum())){
            if(Objects.isNull(comparing)){
                comparing = Comparator.comparing(EduOrganStudentDataDto::getVipStudentNum, "ASC".equals(queryInfo.getVipStudentNum())?Comparator.naturalOrder():Comparator.reverseOrder());
            }else{
                comparing.thenComparing(EduOrganStudentDataDto::getVipStudentNum, "ASC".equals(queryInfo.getVipStudentNum())?Comparator.naturalOrder():Comparator.reverseOrder());
            }
        }
        if(StringUtils.isNotBlank(queryInfo.getVipStudentDuty())){
            if(Objects.isNull(comparing)){
                comparing = Comparator.comparing(EduOrganStudentDataDto::getVipStudentDuty, "ASC".equals(queryInfo.getVipStudentDuty())?Comparator.naturalOrder():Comparator.reverseOrder());
            }else{
                comparing.thenComparing(EduOrganStudentDataDto::getVipStudentDuty, "ASC".equals(queryInfo.getVipStudentDuty())?Comparator.naturalOrder():Comparator.reverseOrder());
            }
        }
        if(StringUtils.isNotBlank(queryInfo.geteVipStudentNum())){
            if(Objects.isNull(comparing)){
                comparing = Comparator.comparing(EduOrganStudentDataDto::geteVipStudentNum, "ASC".equals(queryInfo.geteVipStudentNum())?Comparator.naturalOrder():Comparator.reverseOrder());
            }else{
                comparing.thenComparing(EduOrganStudentDataDto::geteVipStudentNum, "ASC".equals(queryInfo.geteVipStudentNum())?Comparator.naturalOrder():Comparator.reverseOrder());
            }
        }
        if(StringUtils.isNotBlank(queryInfo.getNewMemberStudentNum())){
            if(Objects.isNull(comparing)){
                comparing = Comparator.comparing(EduOrganStudentDataDto::getNewMemberStudentNum, "ASC".equals(queryInfo.getNewMemberStudentNum())?Comparator.naturalOrder():Comparator.reverseOrder());
            }else{
                comparing.thenComparing(EduOrganStudentDataDto::getNewMemberStudentNum, "ASC".equals(queryInfo.getNewMemberStudentNum())?Comparator.naturalOrder():Comparator.reverseOrder());
            }
        }
        if(StringUtils.isNotBlank(queryInfo.getCloudStudyLivelyStudentDuty())){
            if(Objects.isNull(comparing)){
                comparing = Comparator.comparing(EduOrganStudentDataDto::getCloudStudyLivelyStudentDuty, "ASC".equals(queryInfo.getCloudStudyLivelyStudentDuty())?Comparator.naturalOrder():Comparator.reverseOrder());
            }else{
                comparing.thenComparing(EduOrganStudentDataDto::getCloudStudyLivelyStudentDuty, "ASC".equals(queryInfo.getCloudStudyLivelyStudentDuty())?Comparator.naturalOrder():Comparator.reverseOrder());
            }
        }
        if(StringUtils.isNotBlank(queryInfo.getNewStudentNum())){
            if(Objects.isNull(comparing)){
                comparing = Comparator.comparing(EduOrganStudentDataDto::getNewStudentNum, "ASC".equals(queryInfo.getNewStudentNum())?Comparator.naturalOrder():Comparator.reverseOrder());
            }else{
                comparing.thenComparing(EduOrganStudentDataDto::getNewStudentNum, "ASC".equals(queryInfo.getNewStudentNum())?Comparator.naturalOrder():Comparator.reverseOrder());
            }
        }
        if(StringUtils.isNotBlank(queryInfo.getNewStudentDuty())){
            if(Objects.isNull(comparing)){
                comparing = Comparator.comparing(EduOrganStudentDataDto::getNewStudentDuty, "ASC".equals(queryInfo.getNewStudentDuty())?Comparator.naturalOrder():Comparator.reverseOrder());
            }else{
                comparing.thenComparing(EduOrganStudentDataDto::getNewStudentDuty, "ASC".equals(queryInfo.getNewStudentDuty())?Comparator.naturalOrder():Comparator.reverseOrder());
            }
        }

        if(Objects.isNull(comparing)){
            comparing = Comparator.comparing(EduOrganStudentDataDto::getOrganId);
        }

        result = result.stream().sorted(comparing).skip(pageInfo.getOffset()).limit(pageInfo.getLimit()).collect(Collectors.toList());
        AtomicInteger i= new AtomicInteger();
        result.forEach(e->e.setIndex(i.incrementAndGet()));
        pageInfo.setRows(result);
        return pageInfo;
    }

    @Override
    public CloudStudyStudentDataDto getCloudStudyStudentOverView(List<Integer> organIds) {
        CloudStudyStudentDataDto result = new CloudStudyStudentDataDto();
        result.setTotalStudentNum(studentDao.getOrgansTotalStudentNum(organIds));
        result.setVipStudentNum(cloudTeacherDao.getOrgansTotalVipStudentNum(organIds));
        result.seteVipStudentNum(studentDao.getOrgansTotalEVipStudentNum(organIds));
        result.setCloudStudyUseStudentNum(studentDao.getOrgansTotalCloudStudyStudentNum(organIds));
        result.setCloudStudyTodayUseStudentNum(studentDao.getOrgansTodayTotalCloudStudyStudentNum(organIds));
        result.setCloudStudyLivelyStudentNum(studentDao.getOrganTotalCloudStudyLivelyStudentNum(organIds));
        result.setNewCloudStudyStudentNum(sysMusicCompareRecordDao.getOrgansTotalNewCloudStudyNum(organIds));
        if(result.getVipStudentNum()>0&&result.getTotalStudentNum()>0){
            result.setVipStudentDuty(new BigDecimal(result.getVipStudentNum()).divide(new BigDecimal(result.getTotalStudentNum()), 4, BigDecimal.ROUND_DOWN).multiply(new BigDecimal(100)).floatValue());
        }
        return result;
    }

    @Override
    public void cleanStudentCloudStudySequenceDays() {
        studentDao.cleanStudentCloudStudySequenceDays();
    }
}
