package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.StudentDao;
import com.ym.mec.biz.dal.dao.SysMusicCompareRecordDao;
import com.ym.mec.biz.dal.dao.TeacherDao;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.IndexBaseMonthData;
import com.ym.mec.biz.dal.entity.SysMusicCompareRecord;
import com.ym.mec.biz.dal.enums.FeatureType;
import com.ym.mec.biz.dal.enums.IndexDataType;
import com.ym.mec.biz.dal.page.SysMusicCompareRecordQueryInfo;
import com.ym.mec.biz.service.SysMusicCompareRecordService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import io.swagger.models.auth.In;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.time.DayOfWeek;
import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class SysMusicCompareRecordServiceImpl extends BaseServiceImpl<Long, SysMusicCompareRecord>  implements SysMusicCompareRecordService {
	
	@Autowired
	private SysMusicCompareRecordDao sysMusicCompareRecordDao;
	@Autowired
	private TeacherDao teacherDao;
	@Autowired
	private StudentDao studentDao;

	@Override
	public BaseDAO<Long, SysMusicCompareRecord> getDAO() {
		return sysMusicCompareRecordDao;
	}

	@Override
	public long insert(SysMusicCompareRecord bean) {
		studentDao.addStudentCloudStudySequenceDays(bean.getUserId());
		return super.insert(bean);
	}

	@Override
	public void saveMusicCompareData(String phone, SoundCompareHelper soundCompareInfo) {
		SysUser user = teacherDao.getUserWithPhone(phone);
		if(Objects.isNull(user)){
			return;
		}
		SysMusicCompareRecord sysMusicCompareRecord = new SysMusicCompareRecord();
		sysMusicCompareRecord.setUserId(user.getId());
		sysMusicCompareRecord.setSysMusicScoreId(soundCompareInfo.getMusicScoreId());
		sysMusicCompareRecord.setHeardLevel(soundCompareInfo.getHeardLevel());
		sysMusicCompareRecord.setBehaviorId(soundCompareInfo.getBehaviorId());
		Map<String, Object> scoreData = new HashMap<>();
		scoreData.put("userMeasureScore", soundCompareInfo.getUserMeasureScoreMap());
		Map<String, Object> musicalNotesPlayStats = new HashMap<>();
		musicalNotesPlayStats.put("detailId", soundCompareInfo.getDetailId());
		musicalNotesPlayStats.put("examSongId", soundCompareInfo.getExamSongId());
		musicalNotesPlayStats.put("xmlUrl", soundCompareInfo.getXmlUrl());
		musicalNotesPlayStats.put("notesData", soundCompareInfo.getMusicalNotesPlayStats());
		scoreData.put("musicalNotesPlayStats", musicalNotesPlayStats);
		sysMusicCompareRecord.setScoreData(JSON.toJSONString(scoreData));
		if (Objects.nonNull(soundCompareInfo.getUserMeasureScoreMap()) && soundCompareInfo.getUserMeasureScoreMap().containsKey(-1)){
			Map<String, Object> finalScore = soundCompareInfo.getUserMeasureScoreMap().get(-1);
			sysMusicCompareRecord.setScore((BigDecimal) finalScore.get("score"));
			sysMusicCompareRecord.setIntonation((BigDecimal) finalScore.get("intonation"));
			sysMusicCompareRecord.setCadence((BigDecimal) finalScore.get("cadence"));
			sysMusicCompareRecord.setIntegrity((BigDecimal) finalScore.get("integrity"));
		}
		sysMusicCompareRecord.setRecordFilePath(soundCompareInfo.getRecordFilePath());
		sysMusicCompareRecord.setDeviceType(soundCompareInfo.getDeviceType());
		sysMusicCompareRecord.setClientId(soundCompareInfo.getClientId());
		if(Objects.nonNull(soundCompareInfo.getFile())){
			sysMusicCompareRecord.setPlayTime(soundCompareInfo.getFile().length()/(SoundCompareHandler.soundCompareConfig.audioFormat.getFrameSize()*SoundCompareHandler.soundCompareConfig.audioFormat.getFrameRate()));
		}
		sysMusicCompareRecord.setFeature(FeatureType.CLOUD_STUDY_EVALUATION);
		sysMusicCompareRecordDao.insert(sysMusicCompareRecord);
		studentDao.addStudentCloudStudySequenceDays(user.getId());
	}

	@Override
	public Object getLastEvaluationMusicalNotesPlayStats(Integer userId) {
		SysMusicCompareRecord userLastEvaluationData = sysMusicCompareRecordDao.getUserLastEvaluationData(userId);
		if(StringUtils.isBlank(userLastEvaluationData.getScoreData())){
			return null;
		}
		JSONObject jsonObject = JSON.parseObject(userLastEvaluationData.getScoreData());
		if(!jsonObject.containsKey("musicalNotesPlayStats")){
			return null;
		}
		return jsonObject.get("musicalNotesPlayStats");
	}

	@Override
	public StatDto rankingList(SysMusicCompareRecordQueryInfo queryInfo) {
		StatDto result = new StatDto();
		MusicCompareRankingDto head = new MusicCompareRankingDto();
		head.setUserId(queryInfo.getUserId());
		List<MusicCompareRankingDto> userTrainStat = sysMusicCompareRecordDao.getUserTrainStat(queryInfo.getStartTime(), queryInfo.getEndTime(), queryInfo.getHeardLevel());
		List<MusicCompareRankingDto> detail = new ArrayList<>();
		switch (queryInfo.getOrderType()){
			case 1:
				userTrainStat.sort(Comparator.comparing(MusicCompareRankingDto::getScore, Comparator.reverseOrder()).thenComparing(MusicCompareRankingDto::getUserId));
				break;
			case 2:
				userTrainStat.sort(Comparator.comparing(MusicCompareRankingDto::getTrainDays, Comparator.reverseOrder()).thenComparing(MusicCompareRankingDto::getUserId));
				break;
			default:
				userTrainStat.sort(Comparator.comparing(MusicCompareRankingDto::getTrainTime, Comparator.reverseOrder()).thenComparing(MusicCompareRankingDto::getUserId));
				break;
		}

		List<Integer> studentIds = userTrainStat.stream().limit(10).map(MusicCompareRankingDto::getUserId).collect(Collectors.toList());
		Map<Integer, String> studentSubjectMap = new HashMap<>();
		if(!CollectionUtils.isEmpty(studentIds)){
			List<Map<Integer, String>> studentSubjectMapList = studentDao.getStudentSubjectMapList(studentIds);
			studentSubjectMap = MapUtil.convertIntegerMap(studentSubjectMapList);
		}

		for (int i = 0; i < userTrainStat.size(); i++) {
			userTrainStat.get(i).setRankNum(i+1);
			userTrainStat.get(i).setTrainTime(userTrainStat.get(i).getTrainTime()/60);
			if(studentSubjectMap.containsKey(userTrainStat.get(i).getUserId())){
				userTrainStat.get(i).setSubjectName(studentSubjectMap.get(userTrainStat.get(i).getUserId()));
			}
			if(detail.size()<10){
				detail.add(userTrainStat.get(i));
			}
			if(userTrainStat.get(i).getUserId().equals(queryInfo.getUserId())){
				head = userTrainStat.get(i);
			}
		}
		result.setHead(head);
		detail.sort(Comparator.comparing(MusicCompareRankingDto::getRankNum).thenComparing(MusicCompareRankingDto::getUserId));
		result.setDetail(detail);
		return result;
	}

	@Override
	public Map<String, Object> studentTrainData(SysMusicCompareRecordQueryInfo queryInfo) {
		StudentTrainOverviewDto userTrainOverView = new StudentTrainOverviewDto();

		List<StudentTrainChartDto> userTrainChartData = new ArrayList<>();
		if(queryInfo.getPage()==1){
			userTrainOverView = sysMusicCompareRecordDao.getUserTrainOverView(queryInfo.getUserId());
			userTrainChartData = sysMusicCompareRecordDao.getUserTrainChartData(queryInfo.getUserId(), queryInfo.getStartTime(), queryInfo.getEndTime());
		}

		if(CollectionUtils.isEmpty(userTrainChartData)||userTrainChartData.size()!=7){
			Set<LocalDate> trainDates = userTrainChartData.stream().map(StudentTrainChartDto::getTrainDate).collect(Collectors.toSet());
			LocalDate startLocalDate = LocalDate.parse(queryInfo.getStartTime(), DateUtil.dateFormatter);
			for (int i = 1; i <= 7; i++) {
				if(!trainDates.contains(startLocalDate)){
					userTrainChartData.add(new StudentTrainChartDto(0, 0, startLocalDate));
				}
				startLocalDate=startLocalDate.plusDays(1);
			}
		}

		PageInfo<SysMusicCompareRecord> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<>();
		queryInfo.setFeatureType(FeatureType.CLOUD_STUDY_EVALUATION);
		MapUtil.populateMap(params, queryInfo);

		List<SysMusicCompareRecord> dataList = null;
		int count = this.findCount(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = this.getDAO().queryPage(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		Map<String, Object> result = new HashMap<>();
		userTrainOverView.setTrainTime(userTrainOverView.getTrainTime()/60);
		result.put("userTrainOverView",userTrainOverView);

		List<IndexBaseDto> userTrainChartResult = new ArrayList<>();
		List<IndexBaseMonthData> trainNumData = new ArrayList<>();
		List<IndexBaseMonthData> trainTimeData = new ArrayList<>();

		for (StudentTrainChartDto data : userTrainChartData) {
			Date trainDate = Date.from(data.getTrainDate().atStartOfDay(DateUtil.zoneId).toInstant());
			trainTimeData.add(new IndexBaseMonthData(trainDate, null, new BigDecimal(data.getTrainTime()/60)));
			trainNumData.add(new IndexBaseMonthData(trainDate, null, new BigDecimal(data.getTrainNum())));
		}

		userTrainChartResult.add(new IndexBaseDto(IndexDataType.CLOUD_STUDY_TRAIN_TIME, trainTimeData));
		userTrainChartResult.add(new IndexBaseDto(IndexDataType.CLOUD_STUDY_TRAIN_NUM, trainNumData));

		result.put("userTrainChartData", userTrainChartResult);
		result.put("detail",pageInfo);
		return result;
	}

	@Override
	public PageInfo<MusicCompareRankingDto> queryMusicGroupStudentTrainData(SysMusicCompareRecordQueryInfo queryInfo) {
		PageInfo<MusicCompareRankingDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);

		List<MusicCompareRankingDto> dataList = new ArrayList<>();
		int count = sysMusicCompareRecordDao.countMusicGroupStudentTrainData(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList =sysMusicCompareRecordDao.queryMusicGroupStudentTrainData(params);
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public PageInfo<SysMusicCompareRecord> queryStudentTrainData(SysMusicCompareRecordQueryInfo queryInfo) {
		PageInfo<SysMusicCompareRecord> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);

		List<SysMusicCompareRecord> dataList = null;
		int count = this.findCount(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = this.getDAO().queryPage(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

    @Override
    public PageInfo<CountStudentTrainDataDto> countStudentTrain(SysMusicCompareRecordQueryInfo queryInfo) {
		PageInfo<CountStudentTrainDataDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);

		List<CountStudentTrainDataDto> dataList = new ArrayList<>();
		int count = sysMusicCompareRecordDao.countStudentTrain(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList =sysMusicCompareRecordDao.queryStudentTrain(params);
		}
		pageInfo.setRows(dataList);
		return pageInfo;
    }
}
